#!/bin/bash

ARGSPEC="--rows:ROWS --cols:COLS --num-teams:NUM_TEAMS \
--team-size:TEAM_SIZE --deadline:DEADLINE \
--prop:PROPERTY --comp:COMP --det-cap:DET_CAP \
--def-cap:DEF_CAP --loc-cap:LOC_CAP --p-detect-mine:P_DETECT_MINE \
--p-edet-sc0:P_ED_SC0 --p-edet-sc1:P_ED_SC1 --p-edet-sc2:P_ED_SC2 \
--p-fn-sc0:P_FN_SC0 --p-fn-sc1:P_FN_SC1 --p-fn-sc2:P_FN_SC2 \
--p-edef-dc0:P_ED_DC0 --p-edef-dc1:P_ED_DC1 --p-edef-dc2:P_ED_DC2 \
--p-def1:P_DEF1 --p-def2:P_DEF2 --p-def3:P_DEF3 \
--p-skip-lc0:P_SKIP_LC0 --p-skip-lc1:P_SKIP_LC1 --p-skip-lc2:P_SKIP_LC2 \
--p-turn-lc0:P_TURN_LC0 --p-turn-lc1:P_TURN_LC1 --p-turn-lc2:P_TURN_LC2 \
--p-elect-leader:P_ELECT_LEADER"

#check arguments
function usage {
    echo "Usage : $0 <options-file>"
    echo "Each line of options-file should be one of:"
    for X in $ARGSPEC; do
        echo "     $X"
    done
    exit 1
}

#get start time
START=$(date +%s.%N)

#check arguments
if [ "$#" != "1" ]; then
    usage
fi

#parse options
for i in `cat $1`; do
    Y1=`echo $i | awk -F':' '{print $1}'`
    Y2=`echo $i | awk -F':' '{print $2}'`
    for X in $ARGSPEC; do
        X1=`echo $X | awk -F':' '{print $1}'`
        X2=`echo $X | awk -F':' '{print $2}'`
        if [ "$X1" == "$Y1" ]; then
            eval ${X2}=$Y2
            shift 2
            break
        fi
    done
done

#check all options defined
for X in $ARGSPEC; do
    X1=`echo $X | awk -F':' '{print $1}'`
    X2=`echo $X | awk -F':' '{print $2}'`
    if [ -z "${!X2}" ]; then
        echo "ERROR: ${X1} undefined"
        usage
    fi
    echo "BRUNCH_STAT $X2 ${!X2}"
done

CONST="P_ED_SC0=$P_ED_SC0,P_ED_SC1=$P_ED_SC1,P_ED_SC2=$P_ED_SC2,\
P_FN_SC0=$P_FN_SC0,P_FN_SC1=$P_FN_SC1,P_FN_SC2=$P_FN_SC2,\
P_ED_DC0=$P_ED_DC0,P_ED_DC1=$P_ED_DC1,P_ED_DC2=$P_ED_DC2,\
DETECTING_CAPABILITY=$DET_CAP,\
DEFUSING_CAPABILITY=$DEF_CAP,\
LOCOMOTION_CAPABILITY=$LOC_CAP,\
P_DETECT_MINE=$P_DETECT_MINE,\
P_IN_DEFUSE1=$P_DEF1,P_IN_DEFUSE2=$P_DEF2,P_IN_DEFUSE3=$P_DEF3,\
P_SKIP_LC0=$P_SKIP_LC0,P_SKIP_LC1=$P_SKIP_LC1,P_SKIP_LC2=$P_SKIP_LC2,\
P_TURN_LC0=$P_TURN_LC0,P_TURN_LC1=$P_TURN_LC1,P_TURN_LC2=$P_TURN_LC2,\
P_ELECT_LEADER=$P_ELECT_LEADER"

#compositional
if [ "$COMP" == "yes" ]; then
    RES=""
    for i in `seq 1 $NUM_TEAMS`; do
        TMPD=`mktemp -d /tmp/comp-verify.XXXXXXXX`
        echo "=== using temporary directory $TMPD"
        PM_FILE="$TMPD/demining.pm"
        PCTL_FILE="$TMPD/demining.pctl"
        ./demining-gen.sh --pm-file $PM_FILE --pctl-file $PCTL_FILE --rows $ROWS \
            --cols $COLS --num-teams $NUM_TEAMS --first-team $i --last-team $i \
            --team-size $TEAM_SIZE --deadline $DEADLINE
        echo "%%%%%% prism $PM_FILE $PCTL_FILE -prop $PROPERTY -const $CONST"
        prism $PM_FILE $PCTL_FILE -prop $PROPERTY -const $CONST 2>&1 | tee $TMPD/prism.out

        #get local result
        LOCRES=`grep Result $TMPD/prism.out | grep "value in the initial state" | awk '{print $2}'`
        LOCRES=${LOCRES/[eE][+]/*10^}
        LOCRES=${LOCRES/[eE][-]/*10^-}
        echo "%%%%%% result for component $i = $LOCRES"
        if [ "$PROPERTY" == "success_prob" ]; then
            if [ "$RES" == "" ]; then
                RES="$LOCRES"
            else
                RES=`echo "$RES * $LOCRES" | bc -l`
            fi
        else
            if [ "$RES" == "" ]; then
                RES="$LOCRES"
            else
                RES=`echo "$RES + $LOCRES" | bc -l`
            fi
        fi
        rm -fr $TMPD
    done
    echo "%%%%%% overall result = $RES"
    echo "BRUNCH_STAT RESULT $RES"
else
    TMPD=`mktemp -d /tmp/comp-verify.XXXXXXXX`
    echo "=== using temporary directory $TMPD"
    PM_FILE="$TMPD/demining.pm"
    PCTL_FILE="$TMPD/demining.pctl"
    ./demining-gen.sh --pm-file $PM_FILE --pctl-file $PCTL_FILE --rows $ROWS \
        --cols $COLS --num-teams $NUM_TEAMS --first-team 1 --last-team $NUM_TEAMS \
        --team-size $TEAM_SIZE --deadline $DEADLINE
    prism $PM_FILE $PCTL_FILE -prop $PROPERTY -const $CONST 2>&1 | tee $TMPD/prism.out
    RES=`grep Result $TMPD/prism.out | grep "value in the initial state" | awk '{print $2}'`
    RES=${RES/[eE][+]/*10^}
    RES=${RES/[eE][-]/*10^-}
    echo "%%%%%% overall result = $RES"
    echo "BRUNCH_STAT RESULT $RES"
fi

#get end time and print difference
END=$(date +%s.%N)
DIFF=$(echo "scale=0; $END/1 - $START/1" | bc)
echo "%%%%%% total time taken = $DIFF"
echo "BRUNCH_STAT TIME $DIFF"
