#!/bin/bash

TIMEOUT=1800
IDIR="spin13-input-files"
ODIR="spin13-output-files"

#create directory for input and output files
if [ ! -e $IDIR ]; then
    mkdir $IDIR
fi
if [ ! -e $ODIR ]; then
    mkdir $ODIR
fi

NUM_TEAMS=""
TEAM_SIZE=""
PROP=""
COMP=""
DET_CAP=""
DEF_CAP=""
LOC_CAP=""

function create_comp_verify_in {
cat <<EOF > $1
--rows:10 
--cols:12
--num-teams:$NUM_TEAMS
--team-size:$TEAM_SIZE
--deadline:250
--prop:$PROP 
--comp:$COMP
--det-cap:$DET_CAP
--def-cap:$DEF_CAP
--loc-cap:$LOC_CAP
--p-detect-mine:0.5
--p-edet-sc0:0.05
--p-edet-sc1:0.04
--p-edet-sc2:0.03
--p-fn-sc0:0.05
--p-fn-sc1:0.01
--p-fn-sc2:0.005
--p-edef-dc0:0.05
--p-edef-dc1:0.04
--p-edef-dc2:0.03
--p-def1:0.25
--p-def2:0.33
--p-def3:0.5
--p-skip-lc0:0.9999
--p-skip-lc1:0.99999
--p-skip-lc2:0.999999
--p-turn-lc0:0.9999
--p-turn-lc1:0.99999
--p-turn-lc2:0.999999
--p-elect-leader:0.9
EOF
}

#experiments with 2 teams, varying team sizes
function exp_var_team_sizes {
    EXPNAME="$1"
    PROP="$2"
    COMP="$3"
    for DET_CAP in 0 2; do
        for DEF_CAP in 0 2; do
            for LOC_CAP in 0 2; do
                EXP="$EXPNAME-comp-$COMP-det-$DET_CAP-def-$DEF_CAP-loc-$LOC_CAP"
                if [ -e $ODIR/brunch.$EXP ]; then
                    echo "===== $ODIR/brunch.$EXP exists ..."
                    echo "===== skipping $EXP ..."
                else
                    echo "===== starting $EXP"
                    for TEAM_SIZE in 2 3 5 10 15; do
                        printf -v TS "%03d" $TEAM_SIZE
                        NUM_TEAMS="2"
                        INFILE="$IDIR/${EXP}-team-size-$TS"
                        create_comp_verify_in $INFILE 
                    done
                    ./brunch --cpu 1800 --mem 1024 --out $ODIR/brunch.$EXP \
                        --format File:TIME:NUM_TEAMS:TEAM_SIZE:RESULT:Status $IDIR/$EXP-* \
                        -- comp-verify.sh
                fi
                #create csv file
                cat $ODIR/brunch.$EXP/stats  | awk -F',' \
                    '{if (NR > 1) printf("%s,%d,%d,%d,%0.3f\n",$1,$2,$3,$4,$5); else print $0}' \
                    > $ODIR/brunch.$EXP/stats.csv
            done
        done
    done
}

#experiments with 30 robots, varying number of teams and team sizes
function exp_const_num_robots {
    EXPNAME="$1"
    PROP="$2"
    COMP="$3"
    for DET_CAP in 0 2; do
        for DEF_CAP in 0 2; do
            for LOC_CAP in 0 2; do
                EXP="$EXPNAME-comp-$COMP-det-$DET_CAP-def-$DEF_CAP-loc-$LOC_CAP"
                if [ -e $ODIR/brunch.$EXP ]; then
                    echo "===== $ODIR/brunch.$EXP exists ..."
                    echo "===== skipping $EXP ..."
                else
                    echo "===== starting $EXP"
                    for NUM_TEAMS in 003 006 010 015 030; do
                        TEAM_SIZE=`echo "scale=0; 30 / $NUM_TEAMS" | bc -l`
                        INFILE="$IDIR/${EXP}-num-teams-$NUM_TEAMS"
                        create_comp_verify_in $INFILE 
                    done
                    ./brunch --cpu 1800 --mem 1024 --out $ODIR/brunch.$EXP \
                        --format File:TIME:NUM_TEAMS:TEAM_SIZE:RESULT:Status $IDIR/$EXP-* \
                        -- comp-verify.sh
                fi
                #create csv file
                cat $ODIR/brunch.$EXP/stats  | awk -F',' \
                    '{if (NR > 1) printf("%s,%d,%d,%d,%0.3f\n",$1,$2,$3,$4,$5); else print $0}' \
                    > $ODIR/brunch.$EXP/stats.csv
            done
        done
    done
}

#monolithic experiments
if [ "$1" == "--mono" ]; then
    exp_var_team_sizes success1 success_prob no
    exp_const_num_robots success2 success_prob no
    exp_var_team_sizes coverage1 expected_covered no
    exp_const_num_robots coverage2 expected_covered no
else
#compositional experiments
    exp_var_team_sizes success1 success_prob yes
    exp_const_num_robots success2 success_prob yes
    exp_var_team_sizes coverage1 expected_covered yes
    exp_const_num_robots coverage2 expected_covered yes
    ./create-csvs.sh $ODIR
fi
